try:
    from asreviewcontrib.datatools._version import __version__
    from asreviewcontrib.datatools._version import __version_tuple__
except ImportError:
    __version__ = "0.0.0"
    __version_tuple__ = (0, 0, 0)
